# -*- coding: utf-8 -*-

""" LomWriter
Some more in-depth information on how this class works.

Different type of methodes:
 __setField* sets the individual fields
 __check*Element validates the different kinds of input that should be supported for that element and returns element, single or in list
 __setField, actually sets the returned element or list of elements
 __get*Element return the actual etree parts
"""

from pylom.lom import Lom
from lxml import etree

class LomWriter(Lom):
    def __init__(self,language="en"):
        Lom.__init__(self,language)
        self.lomns = '{http://www.imsglobal.org/xsd/imsmd_v1p2}'
        self.lomxml = etree.Element(self.lomns + 'lom', nsmap={"lom": 'http://www.imsglobal.org/xsd/imsmd_v1p2', "xsi": "http://www.w3.org/2001/XMLSchema-instance"})
        self.lom = ""

        # these will not be appended if they have no children
        self.general = etree.Element(self.lomns + "general")
        self.lifecycle = etree.Element(self.lomns + "lifecycle")
        self.metametadata = etree.Element(self.lomns + "metametadata")
        self.technical = etree.Element(self.lomns + "technical")
        self.educational = []
        self.rights = etree.Element(self.lomns + "rights")
        self.relation = []
        self.classification = []

        # used in educational, relation and classification
        self.tmp_element = None

        # python 2/3 compatible basestring instance checking
        try:
            basestring
            self.basestring = basestring
        except NameError:
            self.basestring = str


    def parseDict(self,lomdict):
        """ Parses dict into LOM string and sets this in the lom attribute. """
        for fieldkey in self.fmethods:
            if fieldkey in lomdict:
                attribute = "_" + self.__class__.__name__ + self.fmethods[fieldkey]
                getattr(self,attribute)(lomdict[fieldkey])

        self.__compileLom()


    def __compileLom(self):
        """ Aggregates all the main subelements and writes object to string. """
        if self.general.getchildren():
            self.lomxml.append(self.general)
        if self.lifecycle.getchildren():
            self.lomxml.append(self.lifecycle)
        if self.metametadata.getchildren():
            self.lomxml.append(self.metametadata)
        if self.technical.getchildren():
            self.lomxml.append(self.technical)
        for e in self.educational:
            if e.getchildren():
                self.lomxml.append(e)
        if self.rights.getchildren():
            self.lomxml.append(self.rights)
        for e in self.relation:
            self.lomxml.append(e)
        for e in self.classification:
            self.lomxml.append(e)

        self.lom = etree.tostring(self.lomxml, pretty_print=True)


    def __setFieldTitle(self,value):
        self.__setField(self.general,self.__checkLangstringElement("title",value))

    def __setFieldCatalogEntry(self,value):
        for v in value:
            self.__setField(self.general,self.__checkCatalogEntryElement(v))

    def __setFieldLanguage(self,value):
        self.__setField(self.general,self.__checkElement("language",value))

    def __setFieldDescription(self,value):
        self.__setField(self.general,self.__checkLangstringElement("description",value))

    def __setFieldKeyword(self,value):
        self.__setField(self.general,self.__checkLangstringElement("keyword",value))

    def __setFieldCoverage(self,value):
        self.__setField(self.general,self.__checkLangstringElement("coverage",value))

    def __setFieldStructure(self,value):
        self.__setField(self.general,self.__checkVocabularyElement("structure",value))

    def __setFieldAggregationLevel(self,value):
        self.__setField(self.general,self.__checkVocabularyElement("aggregationlevel",value))

    def __setFieldVersion(self,value):
        self.__setField(self.lifecycle,self.__checkLangstringElement("version",value,"x-none"))

    def __setFieldStatus(self,value):
        self.__setField(self.lifecycle,self.__checkVocabularyElement("status",value))

    def __setFieldContribute(self,value):
        for v in value:
           self.__setContributeElement(self.lifecycle,v)

    def __setFieldMetaCatalogEntry(self,value):
        for v in value:
            self.__setField(self.metametadata,self.__checkCatalogEntryElement(v))

    def __setFieldMetaContribute(self,value):
        for v in value:
           self.__setContributeElement(self.metametadata,v)

    def __setFieldMetadataScheme(self,value):
        self.__setField(self.metametadata,self.__checkElement("metadatascheme",value))

    def __setFieldMetaLanguage(self,value):
        self.__setField(self.metametadata,self.__checkElement("language",value))

    def __setFieldFormat(self,value):
        self.__setField(self.technical,self.__checkElement("format",value))

    def __setFieldSize(self,value):
        self.__setField(self.technical,self.__checkElement("size",value))

    def __setFieldLocation(self,value):
        self.__setField(self.technical,self.__checkElement("location",value))

    def __setFieldDuration(self,value):
        self.__setField(self.technical,self.__checkDateTime("duration",value))

    def __setFieldEducational(self,value):
        for v in value:
            self.tmp_element = self.__getElement("educational")

            if "interactivitytype" in v:
                self.__setField(self.tmp_element, self.__checkVocabularyElement("interactivitytype",v["interactivitytype"]))
            if "learningresourcetype" in v:
                self.__setField(self.tmp_element, self.__checkVocabularyElement("learningresourcetype",v["learningresourcetype"]))
            if "interactivitylevel" in v:
                self.__setField(self.tmp_element, self.__checkVocabularyElement("interactivitylevel",v["interactivitylevel"]))
            if "semanticdensity" in v:
                self.__setField(self.tmp_element, self.__checkVocabularyElement("semanticdensity",v["semanticdensity"]))
            if "intendedenduserrole" in v:
                self.__setField(self.tmp_element, self.__checkVocabularyElement("intendedenduserrole",v["intendedenduserrole"]))
            if "context" in v:
                self.__setField(self.tmp_element, self.__checkVocabularyElement("context",v["context"]))
            if "typicalagerange" in v:
                self.__setField(self.tmp_element, self.__checkLangstringElement("typicalagerange",v["typicalagerange"],"x-none"))
            if "difficulty" in v:
                self.__setField(self.tmp_element, self.__checkVocabularyElement("difficulty",v["difficulty"]))
            if "typicallearningtime" in v:
                self.__setField(self.tmp_element, self.__checkDateTime("typicallearningtime",v["typicallearningtime"]))
            if "description" in v:
                self.__setField(self.tmp_element, self.__checkVocabularyElement("learningresourcetype",v["learningresourcetype"]))
            if "language" in v:
                self.__setField(self.tmp_element, self.__checkElement("language",v["language"]))

            self.educational.append(self.tmp_element)
            self.tmp_element = None

    def __setFieldCost(self,value):
        self.__setField(self.rights,self.__checkVocabularyElement("cost",value))

    def __setFieldCopyrightAndOtherRestrictions(self,value):
        self.__setField(self.rights,self.__checkVocabularyElement("copyrightandotherrestrictions",value))

    def __setFieldCopyrightDescription(self,value):
        self.__setField(self.rights,self.__checkLangstringElement("description",value))

    def __setFieldRelation(self,value):
        for v in value:
            if "kind" not in v or "resource" not in v:
                raise ValueError("bad definition for input field: relation")

            self.tmp_element = self.__getElement("relation")
            self.__setField(self.tmp_element, self.__checkVocabularyElement("kind",v["kind"]))

            if not isinstance(v["resource"], dict) or not "catalogentry" in v["resource"]:
                raise ValueError("bad definition for input field: relation resource")

            resource = self.__getElement("resource")
            if "description" in v["resource"]:
                description = self.__checkLangstringElement("description", v["resource"]["description"])
                if isinstance(description,list):
                    for e in description:
                        resource.append(e)
                else:
                    resource.append(description)

            for c in v["resource"]["catalogentry"]:
                resource.append(self.__checkCatalogEntryElement(c))

            self.tmp_element.append(resource)
            self.relation.append(self.tmp_element)
            self.tmp_element = None

    def __setFieldClassification(self,value):
        for v in value:
            if "purpose" not in v or "taxonpath" not in v:
                raise ValueError("bad definition for input field: classification")

            self.tmp_element = self.__getElement("classification")
            self.__setField(self.tmp_element, self.__checkVocabularyElement("purpose",v["purpose"]))
            if not isinstance(v["taxonpath"], list):
                raise ValueError("bad definition for input field: classification taxonpath")

            for tp in v["taxonpath"]:
                if "source" not in tp or "taxon" not in tp:
                    raise ValueError("bad definition for input field: classification taxonpath")

                taxonpath = self.__getElement("taxonpath")

                if not isinstance(tp["source"], str) or not tp["source"]:
                    raise ValueError("bad definition for input field: classification taxonpath source")
                taxonpath.append(self.__getLangstringElement("source", tp["source"]))

                if not isinstance(tp["taxon"], list):
                    raise ValueError("bad definition for input field: classification taxonpath taxon")

                for t in tp["taxon"]:
                    if not self.__validateTaxon(t):
                        raise ValueError("bad definition for input field: classification taxonpath taxon")

                    taxon = self.__getElement("taxon")
                    taxon.append(self.__getElement("id", t["id"]))
                    taxon.append(self.__getLangstringElement("entry", t["entry"]))
                    taxonpath.append(taxon)

                self.tmp_element.append(taxonpath)

            self.classification.append(self.tmp_element)
            self.tmp_element = None


    def __setField(self,root,element):
        """ Append element; single or from list. """
        if isinstance(element,list):
            for e in element:
                root.append(e)
        else:
            root.append(element)


    def __checkElement(self,field,value):
        if isinstance(value, self.basestring) and value:
            return self.__getElement(field,value)
        elif isinstance(value, list):
            l = []
            for v in (v for v in value if v):
                l.append(self.__getElement(field,v))
            return l
        else:
            raise ValueError("bad definition for input field: " + field)

    def __checkLangstringElement(self,field,value,language=None):
        if isinstance(value, self.basestring) and value:
            return self.__getLangstringElement(field,value,language)
        elif isinstance(value, list):
            l = []
            for v in (v for v in value if v):
                l.append(self.__getLangstringElement(field,v,language))
            return l
        else:
            raise ValueError("bad definition for input field: " + field)

    def __checkCatalogEntryElement(self,value):
        if isinstance(value, dict) and self.__validateCatalogEntry(value):
            return self.__getCatalogEntryElement(value)
        else:
            raise ValueError("bad definition for input field: catalogentry")

    def __checkVocabularyElement(self,field,value,source=None):
        """ Parses value to one or more vocabulary elements.
        vocabulary values can be string, add one and lookup source,
        list of values and lookup source as well,
        or dict, or list of dicts, specifying the source. """
        if isinstance(value, self.basestring) and value:
            return self.__getVocabularyElement(field,value,source)
        elif isinstance(value, dict) and self.__validateVocabulary(value):
            return self.__getVocabularyElement(field,value["value"],value["source"])
        elif isinstance(value, list) and value:
            l = []
            if isinstance(value[0], self.basestring):
                for v in (v for v in value if v):
                    l.append(self.__getVocabularyElement(field,v,source))
            else:
                # TODO support list of source/value dicts
                raise ValueError("bad definition for input field: " + field)
            return l
        else:
            raise ValueError("bad definition for input field: " + field)

    def __checkDateTime(self,field,value):
        if isinstance(value, str) and value:
            date = self.__getElement(field)
            date.append(self.__getElement("datetime",value))
            return date
        elif isinstance(value, dict) and self.__validateDateTime(value):
            date = self.__getElement(field)
            date.append(self.__getElement("datetime",value["datetime"]))
            date.append(self.__getLangstringElement("description",value["description"]))
            return date
        else:
            raise ValueError("bad definition for input field: date")


    def __setContributeElement(self,root,value):
        # initial input validation
        if "role" not in value or ("entity" not in value and "date" not in value):
            raise ValueError("bad definition for input field: contribute, provide a role and an entity or date")

        contribute = self.__getElement("contribute")
        role_default_source = self.vocabulary_sources[self.__getElementName(root) + "-contribute-role"]
        contribute.append(self.__checkVocabularyElement("role",value["role"],role_default_source))

        if "entity" in value:
            if isinstance(value["entity"], str) and value["entity"]:
                entity = self.__getElement("centity")
                entity.append(self.__getElement("vcard",value["entity"]))
                contribute.append(entity)
            elif isinstance(value["entity"], list):
                for v in (v for v in value["entity"] if v):
                    entity = self.__getElement("centity")
                    entity.append(self.__getElement("vcard",value["entity"]))
                    contribute.append(entity)
            else:
                raise ValueError("bad definition for input field: contribute entity")

        if "date" in value:
            contribute.append(self.__checkDateTime("date",value["date"]))

        root.append(contribute)


    def __getElement(self,element,value=None):
        """ Returns a basic xml etree element, empty if no value. """
        e = etree.Element(self.lomns + element)
        if value:
            e.text = value
        return e

    def __getLangstringElement(self,element,value,language=None):
        if not language:
            language = self.lang

        e = etree.Element(self.lomns + element)
        lang = etree.Element(self.lomns + "langstring", {'{http://www.w3.org/XML/1998/namespace}lang':language})
        lang.text = value
        e.append(lang)
        return e

    def __getCatalogEntryElement(self,value):
        e = etree.Element(self.lomns + "catalogentry")
        e.append(self.__getElement("catalog",value["catalog"]))
        e.append(self.__getLangstringElement("entry",value["entry"],"x-none"))
        return e

    def __getVocabularyElement(self,element,value,source=None):
        if not source:
            source = self.vocabulary_sources[element]

        e = etree.Element(self.lomns + element)
        e.append(self.__getLangstringElement("source",source,"x-none"))
        e.append(self.__getLangstringElement("value",value,"x-none"))
        return e


    def __validateCatalogEntry(self,value):
        """ Validates provided dict in __setCatalogEntryElement. """
        if self.__validateDictionaryKey(value,"catalog") and self.__validateDictionaryKey(value,"entry"):
            return True
        else:
            return False
 
    def __validateVocabulary(self,value):
        """ Validates provided value in __setVocabularyElement. """
        if self.__validateDictionaryKey(value,"source") and self.__validateDictionaryKey(value,"value"):
            return True
        else:
            return False

    def __validateDateTime(self,value):
        """ Validates provided value in for a datetime. """
        if self.__validateDictionaryKey(value,"datetime") and self.__validateDictionaryKey(value,"description"):
            return True
        else:
            return False

    def __validateTaxon(self,value):
        """ Validates provided value in for a classification taxon. """
        if self.__validateDictionaryKey(value,"id") and self.__validateDictionaryKey(value,"entry"):
            return True
        else:
            return False

    def __validateDictionaryKey(self,value,key):
        """ Base validator used for validating input catalogentries and vocabularies. """
        if key in value and value[key] and isinstance(value[key], str):
            return True
        else:
            return False

    def __getElementName(self,element):
        """ Easy function to return namespaceless element name. """
        nslen = len(self.lomns)
        return element.tag[nslen:]



from unittest import TestCase
from pylom.writer import LomWriter

class FailWriterElementsTestCase(TestCase):
    def setUp(self):
        self.lomwriter = LomWriter()

    def test_empty_element(self):
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"language": ""})

    def test_empty_langstring_element(self):
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"title": ""})

    def test_empty_catalogentry_element(self):
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"catalogentry": [{"catalog": "", "entry": "https://www.wikidata.org/wiki/Q16987908"}]})
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"catalogentry": [{"catalog": "URI", "entry": ""}]})

    def test_missing_catalogentry_element(self):
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"catalogentry": [{}] })
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"catalogentry": [{"entry": "https://www.wikidata.org/wiki/Q16987908"}]})
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"catalogentry": [{"catalog": "URI"}]})

    def test_empty_vocabulary_element(self):
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"aggregationlevel": ""})

    def test_empty_vocabulary_element_dict(self):
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"aggregationlevel": {"source": "", "value": "2"}})
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"aggregationlevel": {"source": "LOMv1.0", "value": ""}})

    def test_missing_vocabulary_element_dict(self):
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"aggregationlevel": {} })
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"aggregationlevel": {"source": "LOMv1.0"}})
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"aggregationlevel": {"value": "2"}})

    def test_empty_datetime_element(self):
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"duration": ""})

    def test_empty_datetime_element_dict(self):
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"duration": {"datetime": "", "description": "technical reading time"}})
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"duration": {"datetime": "PT0H16M", "description": ""}})

    def test_missing_datetime_element_dict(self):
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"duration": {} })
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"duration": {"datetime": "PT0H16M"}})
        with self.assertRaises(ValueError):
            self.lomwriter.parseDict({"duration": {"description": "technical reading time"}})



from unittest import TestCase
from pylom.reader import LomReader
from pylom.writer import LomWriter

class WriterElementsTestCase(TestCase):
    def setUp(self):
        self.lomreader = LomReader()


    def test_title(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"title": "Bloodbath of B-R5RB"})
        self.lomreader.parseString(lomwriter.lom,["title"])
        self.assertEqual(self.lomreader.lom["title"], "Bloodbath of B-R5RB")

    def test_title_unicode(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"title": u"Bloodbath of B-R5RB"})
        self.lomreader.parseString(lomwriter.lom,["title"])
        self.assertEqual(self.lomreader.lom["title"], "Bloodbath of B-R5RB")

    def test_catalogentry(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"catalogentry": [{"catalog": "URI", "entry": "https://www.wikidata.org/wiki/Q16987908"}]})
        self.lomreader.parseString(lomwriter.lom,["catalogentry"])
        self.assertEqual(self.lomreader.lom["catalogentry"][0]["catalog"], "URI")
        self.assertEqual(self.lomreader.lom["catalogentry"][0]["entry"], "https://www.wikidata.org/wiki/Q16987908")

    def test_language(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"language": "en"})
        self.lomreader.parseString(lomwriter.lom,["language"])
        self.assertIn("en",self.lomreader.lom["language"])

    def test_keyword(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"keyword": ["Eve Online", "Halloween War"]})
        self.lomreader.parseString(lomwriter.lom,["keyword"])
        self.assertIn("Eve Online",self.lomreader.lom["keyword"])
        self.assertIn("Halloween War",self.lomreader.lom["keyword"])

    def test_structure(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"structure": "hierarchical"})
        self.lomreader.parseString(lomwriter.lom,["structure"])
        self.assertEqual(self.lomreader.lom["structure"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["structure"]["value"],"hierarchical")

    def test_structure_unicode(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"structure": u"hierarchical"})
        self.lomreader.parseString(lomwriter.lom,["structure"])
        self.assertEqual(self.lomreader.lom["structure"]["value"],"hierarchical")

    def test_aggregationlevel(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"aggregationlevel": {"source": "custom", "value": "5"}})
        self.lomreader.parseString(lomwriter.lom,["aggregationlevel"])
        self.assertEqual(self.lomreader.lom["aggregationlevel"]["source"],"custom")
        self.assertEqual(self.lomreader.lom["aggregationlevel"]["value"],"5")

    def test_status(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"status": "final"})
        self.lomreader.parseString(lomwriter.lom,["status"])
        self.assertEqual(self.lomreader.lom["status"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["status"]["value"],"final")

    def test_contribute(self):
        contributions = [{"role": "publisher", "entity": "BEGIN:VCARD\nFN:Wikipedia\nEND:VCARD", "date": "2014-02-02T15:30:00Z"}]
        lomwriter = LomWriter()
        lomwriter.parseDict({"contribute": contributions})
        self.lomreader.parseString(lomwriter.lom,["contribute"])
        self.assertEqual(self.lomreader.lom["contribute"][0]["role"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["contribute"][0]["role"]["value"],"publisher")
        self.assertEqual(self.lomreader.lom["contribute"][0]["entity"][0],"BEGIN:VCARD\nFN:Wikipedia\nEND:VCARD")
        self.assertEqual(self.lomreader.lom["contribute"][0]["date"]["datetime"],"2014-02-02T15:30:00Z")

    def test_location(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"location": "https://www.wikidata.org/wiki/Q16987908"})
        self.lomreader.parseString(lomwriter.lom,["location"])
        self.assertEqual(self.lomreader.lom["location"],"https://www.wikidata.org/wiki/Q16987908")

    def test_location_unicode(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"location": u"https://www.wikidata.org/wiki/Q16987908"})
        self.lomreader.parseString(lomwriter.lom,["location"])
        self.assertEqual(self.lomreader.lom["location"],"https://www.wikidata.org/wiki/Q16987908")

    def test_duration(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"duration": {"datetime": "PT0H16M", "description": "technical reading time"}})
        self.lomreader.parseString(lomwriter.lom,["duration"])
        self.assertEqual(self.lomreader.lom["duration"]["datetime"],"PT0H16M")
        self.assertEqual(self.lomreader.lom["duration"]["description"],"technical reading time")

    def test_educational(self):
        educationals = [ {"interactivitytype": "expositive",
                          "learningresourcetype": "narrative text",
                          "interactivitylevel": "low",
                          "semanticdensity": "high",
                          "typicalagerange": "12+",
                          "intendedenduserrole": "learner",
                          "context": ["school", "higher education"],
                          "difficulty": "difficult" }]
        lomwriter = LomWriter()
        lomwriter.parseDict({"educational": educationals})
        self.lomreader.parseString(lomwriter.lom,["educational"])
        self.assertEqual(self.lomreader.lom["educational"][0]["interactivitytype"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["interactivitytype"]["value"], "expositive")
        self.assertEqual(self.lomreader.lom["educational"][0]["learningresourcetype"][0]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["learningresourcetype"][0]["value"], "narrative text")
        self.assertEqual(self.lomreader.lom["educational"][0]["interactivitylevel"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["interactivitylevel"]["value"], "low")
        self.assertEqual(self.lomreader.lom["educational"][0]["semanticdensity"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["semanticdensity"]["value"], "high")
        self.assertEqual(self.lomreader.lom["educational"][0]["intendedenduserrole"][0]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["intendedenduserrole"][0]["value"], "learner")
        self.assertEqual(self.lomreader.lom["educational"][0]["context"][0]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["context"][0]["value"], "school")
        self.assertEqual(self.lomreader.lom["educational"][0]["context"][1]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["context"][1]["value"], "higher education")
        self.assertEqual(self.lomreader.lom["educational"][0]["difficulty"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["difficulty"]["value"], "difficult")
        self.assertEqual(self.lomreader.lom["educational"][0]["typicalagerange"][0], "12+")

    def test_cost(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"cost": "no"})
        self.lomreader.parseString(lomwriter.lom,["cost"])
        self.assertEqual(self.lomreader.lom["cost"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["cost"]["value"],"no")

    def test_copyrightandotherrestrictions(self):
        lomwriter = LomWriter()
        lomwriter.parseDict({"copyrightandotherrestrictions": "yes"})
        self.lomreader.parseString(lomwriter.lom,["copyrightandotherrestrictions"])
        self.assertEqual(self.lomreader.lom["copyrightandotherrestrictions"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["copyrightandotherrestrictions"]["value"],"yes")

    def test_relation(self):
        relations = [
            { "kind": "references", 
                "resource": { 
                    "description": "Wikipedia article references", 
                    "catalogentry": [
                        {"catalog": "URI", "entry": "https://www.wired.com/gamelife/2014/02/eve-online-battle-of-b-r/"},
                        {"catalog": "URI", "entry": "http://www.polygon.com/2014/1/30/5360208/Eve-Onlines-Bloodbath"} ] } },
            { "kind": "isreferencedby", 
                "resource": { 
                    "description": "Wikipedia pages linking to this", 
                    "catalogentry": [
                        {"catalog": "URI", "entry": "https://www.wikidata.org/wiki/Q336177"}] } } ]
        lomwriter = LomWriter()
        lomwriter.parseDict({"relation": relations})
        self.lomreader.parseString(lomwriter.lom,["relation"])
        self.assertEqual(self.lomreader.lom["relation"][0]["kind"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["relation"][0]["kind"]["value"], "references")
        self.assertEqual(self.lomreader.lom["relation"][0]["resource"]["description"][0], "Wikipedia article references")
        self.assertEqual(self.lomreader.lom["relation"][0]["resource"]["catalogentry"][0]["catalog"], "URI")
        self.assertEqual(self.lomreader.lom["relation"][0]["resource"]["catalogentry"][0]["entry"], "https://www.wired.com/gamelife/2014/02/eve-online-battle-of-b-r/")
        self.assertEqual(self.lomreader.lom["relation"][0]["resource"]["catalogentry"][1]["catalog"], "URI")
        self.assertEqual(self.lomreader.lom["relation"][0]["resource"]["catalogentry"][1]["entry"], "http://www.polygon.com/2014/1/30/5360208/Eve-Onlines-Bloodbath")
        self.assertEqual(self.lomreader.lom["relation"][1]["kind"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["relation"][1]["kind"]["value"], "isreferencedby")
        self.assertEqual(self.lomreader.lom["relation"][1]["resource"]["description"][0], "Wikipedia pages linking to this")
        self.assertEqual(self.lomreader.lom["relation"][1]["resource"]["catalogentry"][0]["catalog"], "URI")
        self.assertEqual(self.lomreader.lom["relation"][1]["resource"]["catalogentry"][0]["entry"], "https://www.wikidata.org/wiki/Q336177")

    def test_classification(self):
        classifications = [
            { "purpose": "idea",
                "taxonpath": [ { "source": "https://en.wikipedia.org/wiki/Category:Articles", "taxon": [
                    { "id": "Category:Games", "entry": "Games" },
                    { "id": "Category:Space_MMORPGs", "entry": "Space MMORPGs" } ] } ] } ]
        lomwriter = LomWriter()
        lomwriter.parseDict({"classification": classifications})
        self.lomreader.parseString(lomwriter.lom,["classification"])
        self.assertEqual(self.lomreader.lom["classification"][0]["purpose"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["classification"][0]["purpose"]["value"], "idea")
        self.assertEqual(self.lomreader.lom["classification"][0]["taxonpath"][0]["source"], "https://en.wikipedia.org/wiki/Category:Articles")
        self.assertEqual(self.lomreader.lom["classification"][0]["taxonpath"][0]["taxon"][0]["id"], "Category:Games")
        self.assertEqual(self.lomreader.lom["classification"][0]["taxonpath"][0]["taxon"][0]["entry"], "Games")
        self.assertEqual(self.lomreader.lom["classification"][0]["taxonpath"][0]["taxon"][1]["id"], "Category:Space_MMORPGs")
        self.assertEqual(self.lomreader.lom["classification"][0]["taxonpath"][0]["taxon"][1]["entry"], "Space MMORPGs")



# -*- coding: utf-8 -*-

from collections import OrderedDict

class Lom:
    def __init__(self,language):
        # the etree object
        self.lomxml = None
        # the language used in language specific langstring elements
        self.lang = language

        self.__setFieldMethods()
        self.__setVocabularySources()


    def __setFieldMethods(self):
        """ Sets all field methods in correct order,
        indexed with the fieldset key. """
        self.fmethods = OrderedDict()
        self.fmethods["title"] = "__setFieldTitle"
        self.fmethods["catalogentry"] = "__setFieldCatalogEntry"
        self.fmethods["language"] = "__setFieldLanguage"
        self.fmethods["description"] = "__setFieldDescription"
        self.fmethods["keyword"] = "__setFieldKeyword"
        self.fmethods["coverage"] = "__setFieldCoverage"
        self.fmethods["structure"] = "__setFieldStructure"
        self.fmethods["aggregationlevel"] = "__setFieldAggregationLevel"
        self.fmethods["version"] = "__setFieldVersion"
        self.fmethods["status"] = "__setFieldStatus"
        self.fmethods["contribute"] = "__setFieldContribute"
        self.fmethods["metacatalogentry"] = "__setFieldMetaCatalogEntry"
        self.fmethods["metacontribute"] = "__setFieldMetaContribute"
        self.fmethods["metadatascheme"] = "__setFieldMetadataScheme"
        self.fmethods["metalanguage"] = "__setFieldMetaLanguage"
        self.fmethods["format"] = "__setFieldFormat"
        self.fmethods["size"] = "__setFieldSize"
        self.fmethods["location"] = "__setFieldLocation"
        self.fmethods["duration"] = "__setFieldDuration"
        self.fmethods["educational"] = "__setFieldEducational"
        self.fmethods["cost"] = "__setFieldCost"
        self.fmethods["copyrightandotherrestrictions"] = "__setFieldCopyrightAndOtherRestrictions"
        self.fmethods["copyrightdescription"] = "__setFieldCopyrightDescription"
        self.fmethods["relation"] = "__setFieldRelation"
        self.fmethods["classification"] = "__setFieldClassification"

    def __setVocabularySources(self):
        """ Set default field vocabulary sources"""
        self.vocabulary_sources = {
            "aggregationlevel": "LOMv1.0",
            "structure": "LOMv1.0",
            "status": "LOMv1.0",
            "lifecycle-contribute-role": "LOMv1.0",
            "metametadata-contribute-role": "LOMv1.0",
            "interactivitytype": "LOMv1.0",
            "learningresourcetype": "LOMv1.0",
            "interactivitylevel": "LOMv1.0",
            "semanticdensity": "LOMv1.0",
            "intendedenduserrole": "LOMv1.0",
            "context": "LOMv1.0",
            "difficulty": "LOMv1.0",
            "cost": "LOMv1.0",
            "copyrightandotherrestrictions": "LOMv1.0",
            "kind": "LOMv1.0",
            "purpose": "LOMv1.0" }



from unittest import TestCase
from pylom.reader import LomReader

class FailReaderElementsTestCase(TestCase):
    @classmethod
    def setUpClass(self):
        self.lomreader = LomReader()
        self.lomreader.parsePath("test/records/ims-incomplete.xml")

    def test_empty_element(self):
        self.assertIn("",self.lomreader.lom["keyword"])

    def test_empty_vocabulary_element(self):
        self.assertEqual(self.lomreader.lom["aggregationlevel"]["value"],"")

    def test_no_occurence(self):
        self.assertEqual(self.lomreader.lom["version"],"")



# -*- coding: utf-8 -*-

from pylom.lom import Lom
from lxml import etree

class LomReader(Lom):
    def __init__(self,language="en"):
        Lom.__init__(self,language)
        self.ns = {'lom': 'http://www.imsglobal.org/xsd/imsmd_v1p2'}
        self.__setEmptyLom()
        self.fieldset = []


    def parsePath(self,path,fieldset=[]):
        """ Parse LOM XML based on path. """
        try:
            self.lomxml = etree.parse(path)
        except Exception as e:
            raise RuntimeError(e)

        self.fieldset = fieldset
        self.__parseLom()


    def parseString(self,lomstring,fieldset=[]):
        """ Parse LOM XML based on string. """
        try:
            self.lomxml = etree.fromstring(lomstring)
        except Exception as e:
            raise RuntimeError(e)

        self.fieldset = fieldset
        self.__parseLom()


    def setCustomEmptyLom(self,customdict):
        """ Sets a custom empty LOM dictionary. In this, the user can specify
        how answers should be formatted, for example always return the first title."""
        if isinstance(customdict, dict):
            self.lom = customdict
        else:
            raise TypeError("custom empty LOM should be a dictionary")


    def __parseLom(self):
        if self.lomxml is None:
            raise RuntimeError("there is no LOM Etree object")

        if self.fieldset:
            self.__parseFieldset()
        else:
            self.__parseFull()

    def __parseFull(self):
        """ Parses using all __setField* methods. """
        for fieldkey in self.fmethods:
            attribute = "_" + self.__class__.__name__ + self.fmethods[fieldkey]
            getattr(self,attribute)()

    def __parseFieldset(self):
        """ Parses using only the keys provided in fieldset. """
        for fieldkey in self.fmethods:
            if fieldkey in self.fieldset:
                attribute = "_" + self.__class__.__name__ + self.fmethods[fieldkey]
                getattr(self,attribute)()


    def __setEmptyLom(self):
        """ Creates an empty LOM dictionary. An empty string denotes a single
        value, and empty list a multiple value element. """
        self.lom = {
            "title": "",
            "catalogentry": [],
            "language": [],
            "description": [],
            "keyword": [],
            "coverage": [],
            "structure": "",
            "aggregationlevel": "",
            "version": "",
            "status": "",
            "contribute": [],
            "metacatalogentry": [],
            "metacontribute": [],
            "metadatascheme": [],
            "metalanguage": "",
            "format": [],
            "size": "",
            "location": "",
            "duration": "",
            "educational": [],
            "cost": "",
            "copyrightandotherrestrictions": "",
            "copyrightdescription": "",
            "relation": [],
            "classification": [] }


    def __setFieldTitle(self):
        self.__setElement("/lom:lom/lom:general/lom:title/lom:langstring[@xml:lang='" + self.lang + "']", "title")

    def __setFieldCatalogEntry(self):
        self.__setCatalogEntryElement("/lom:lom/lom:general/lom:catalogentry", "catalogentry")

    def __setFieldLanguage(self):
        self.__setElement("/lom:lom/lom:general/lom:language","language")

    def __setFieldDescription(self):
        self.__setElement("/lom:lom/lom:general/lom:description/lom:langstring[@xml:lang='" + self.lang + "']", "description")

    def __setFieldKeyword(self):
        self.__setElement("/lom:lom/lom:general/lom:keyword/lom:langstring[@xml:lang='" + self.lang + "']", "keyword")

    def __setFieldCoverage(self):
        self.__setElement("/lom:lom/lom:general/lom:coverage/lom:langstring[@xml:lang='" + self.lang + "']", "coverage")

    def __setFieldStructure(self):
        self.__setVocabularyElement("/lom:lom/lom:general/lom:structure", "structure")

    def __setFieldAggregationLevel(self):
        self.__setVocabularyElement("/lom:lom/lom:general/lom:aggregationlevel", "aggregationlevel")

    def __setFieldVersion(self):
        self.__setElement("/lom:lom/lom:lifecycle/lom:version/lom:langstring","version")

    def __setFieldStatus(self):
        self.__setVocabularyElement("/lom:lom/lom:lifecycle/lom:status", "status")

    def __setFieldContribute(self):
        self.__setContributeElement("/lom:lom/lom:lifecycle/lom:contribute", "contribute")

    def __setFieldMetaCatalogEntry(self):
        self.__setCatalogEntryElement("/lom:lom/lom:metametadata/lom:catalogentry", "metacatalogentry")

    def __setFieldMetaContribute(self):
        self.__setContributeElement("/lom:lom/lom:metametadata/lom:contribute", "metacontribute")

    def __setFieldMetadataScheme(self):
        self.__setElement("/lom:lom/lom:metametadata/lom:metadatascheme","metadatascheme")

    def __setFieldMetaLanguage(self):
        self.__setElement("/lom:lom/lom:metametadata/lom:language","metalanguage")

    def __setFieldFormat(self):
        self.__setElement("/lom:lom/lom:technical/lom:format","format")

    def __setFieldSize(self):
        self.__setElement("/lom:lom/lom:technical/lom:size","size")

    def __setFieldLocation(self):
        self.__setElement("/lom:lom/lom:technical/lom:location","location")

    def __setFieldDuration(self):
        self.lom["duration"] = self.__getDurationElement(self.lomxml,"/lom:lom/lom:technical/lom:duration")

    def __setFieldEducational(self):
        """ Parses the educational element. """
        element = self.lomxml.xpath("/lom:lom/lom:educational", namespaces=self.ns)
        if element:
            for e in element:
                data = {
                    "interactivitytype": "",
                    "learningresourcetype": [],
                    "interactivitylevel": "",
                    "semanticdensity": "",
                    "intendedenduserrole": [],
                    "context": [],
                    "typicalagerange": [],
                    "difficulty": "",
                    "typicallearningtime": "",
                    "description": [],
                    "language": [] }
                data["interactivitytype"] = self.__getSingleVocabularyElement(e,"lom:interactivitytype")
                data["learningresourcetype"] = self.__getMultipleVocabularyElement(e,"lom:learningresourcetype")
                data["interactivitylevel"] = self.__getSingleVocabularyElement(e,"lom:interactivitylevel")
                data["semanticdensity"] = self.__getSingleVocabularyElement(e,"lom:semanticdensity")
                data["intendedenduserrole"] = self.__getMultipleVocabularyElement(e,"lom:intendedenduserrole")
                data["context"] = self.__getMultipleVocabularyElement(e,"lom:context")
                data["typicalagerange"] = self.__getMultipleElement(e,"lom:typicalagerange/lom:langstring")
                data["difficulty"] = self.__getSingleVocabularyElement(e,"lom:difficulty")
                data["typicallearningtime"] = self.__getDurationElement(e,"lom:typicallearningtime")
                data["description"] = self.__getMultipleElement(e,"lom:description/lom:langstring[@xml:lang='" + self.lang + "']")
                data["language"] = self.__getMultipleElement(e,"lom:language")

                self.lom["educational"].append(data)

    def __setFieldCost(self):
        self.__setVocabularyElement("/lom:lom/lom:rights/lom:cost", "cost")

    def __setFieldCopyrightAndOtherRestrictions(self):
        self.__setVocabularyElement("/lom:lom/lom:rights/lom:copyrightandotherrestrictions", "copyrightandotherrestrictions")

    def __setFieldCopyrightDescription(self):
        self.__setElement("/lom:lom/lom:rights/lom:description/lom:langstring[@xml:lang='" + self.lang + "']", "copyrightdescription")

    def __setFieldRelation(self):
        """ Parses the relation element. """
        element = self.lomxml.xpath("/lom:lom/lom:relation", namespaces=self.ns)
        if element:
            for e in element:
                data = {"kind": "", "resource": ""}
                data["kind"] = self.__getSingleVocabularyElement(e,"lom:kind")
                resource = e.xpath("lom:resource", namespaces=self.ns)
                if resource:
                    res_data = {"description": [], "catalogentry": []}
                    res_data["description"] = self.__getMultipleElement(resource[0],"lom:description/lom:langstring[@xml:lang='" + self.lang + "']")
                    catalogentry = resource[0].xpath("lom:catalogentry", namespaces=self.ns)
                    if catalogentry:
                        res_data["catalogentry"] = self.__getCatalogEntryElement(catalogentry)
                    data["resource"] = res_data

                self.lom["relation"].append(data)

    def __setFieldClassification(self):
        """ Parses the classification element. """
        element = self.lomxml.xpath("/lom:lom/lom:classification", namespaces=self.ns)
        if element:
            for e in element:
                data = {"purpose": "", "taxonpath": []}
                data["purpose"] = self.__getSingleVocabularyElement(e,"lom:purpose")
                taxonpath = e.xpath("lom:taxonpath", namespaces=self.ns)
                if taxonpath:
                    for path in taxonpath:
                        tp_data = {"source": "", "taxon": []}
                        tp_data["source"] = self.__getSingleElement(path, "lom:source/lom:langstring")
                        taxon = path.xpath("lom:taxon", namespaces=self.ns)
                        if taxon:
                            for t in taxon:
                                t_data = {"id": "", "entry": ""}
                                t_data["id"] = self.__getSingleElement(t, "lom:id")
                                t_data["entry"] = self.__getSingleElement(t, "lom:entry/lom:langstring")
                                tp_data["taxon"].append(t_data)
                        data["taxonpath"].append(tp_data)

                self.lom["classification"].append(data)


    def __setElement(self,xpath,lomkey):
        """ Sets the value from the xpath in the specified lomkey. """
        if isinstance(self.lom[lomkey], str):
            self.lom[lomkey] = self.__getSingleElement(self.lomxml,xpath)
        elif isinstance(self.lom[lomkey], list):
            self.lom[lomkey] = self.__getMultipleElement(self.lomxml,xpath)
        else:
            raise LookupError("bad type definition in empty LOM")


    def __setVocabularyElement(self,xpath,lomkey):
        """ Sets the vocabulary source and value from the xpath in the specified lomkey. """
        if isinstance(self.lom[lomkey], str):
            self.lom[lomkey] = self.__getSingleVocabularyElement(self.lomxml,xpath)
        elif isinstance(self.lom[lomkey], list):
            self.lom[lomkey] = self.__getMultipleVocabularyElement(self.lomxml,xpath)
        else:
            raise LookupError("bad type definition in empty LOM")


    def __setCatalogEntryElement(self,xpath,lomkey):
        """ Sets the catalogs and entries from the xpath in the specified lomkey. """
        element = self.lomxml.xpath(xpath, namespaces=self.ns)
        if element:
            self.lom[lomkey] = self.__getCatalogEntryElement(element)


    def __setContributeElement(self,xpath,lomkey):
        """ Sets each contribute from the xpath in a data structure. """
        element = self.lomxml.xpath(xpath, namespaces=self.ns)
        if element:
            for e in element:
                data = {"role": "", "entity": [], "date": ""}
                data["role"] = self.__getSingleVocabularyElement(e,"lom:role")
                data["entity"] = self.__getMultipleElement(e, "lom:centity/lom:vcard")
                data["date"] = self.__getDurationElement(e, "lom:date")

                self.lom[lomkey].append(data)


    def __getDurationElement(self,etreepart,xpath):
        element = etreepart.xpath(xpath, namespaces=self.ns)
        if element:
            data = {"datetime": "", "description": ""}
            data["datetime"] = self.__getSingleElement(element[0],"lom:datetime")
            data["description"] = self.__getSingleElement(element[0],"lom:description/lom:langstring[@xml:lang='" + self.lang + "']")
            return data
        else:
            return ""

    def __getCatalogEntryElement(self,etreepart):
        """ Return all catalogentry elements as a list of dictionaries. """
        catalogentries = []
        for e in etreepart:
            data = {"catalog": "", "entry": ""}
            data["catalog"] = self.__getSingleElement(e,"lom:catalog")
            data["entry"] = self.__getSingleElement(e,"lom:entry/lom:langstring")
            catalogentries.append(data)
        return catalogentries


    def __getSingleVocabularyElement(self,etreepart,xpath):
        element = etreepart.xpath(xpath, namespaces=self.ns)
        if element:
            data = {"source": "", "value": ""}
            data["source"] = self.__getSingleElement(element[0],"lom:source/lom:langstring")
            data["value"] = self.__getSingleElement(element[0],"lom:value/lom:langstring")
            return data
        else:
            return ""

    def __getMultipleVocabularyElement(self,etreepart,xpath):
        data = []
        element = etreepart.xpath(xpath, namespaces=self.ns)
        if element:
            for e in element:
                vocab = {"source": "", "value": ""}
                vocab["source"] = self.__getSingleElement(e,"lom:source/lom:langstring")
                vocab["value"] = self.__getSingleElement(e,"lom:value/lom:langstring")
                data.append(vocab)
        return data

    def __getSingleElement(self,etreepart,xpath):
        element = etreepart.xpath(xpath, namespaces=self.ns)
        if element:
            if element[0].text:
                return element[0].text
            else:
                return ""
        else:
            return ""

    def __getMultipleElement(self,etreepart,xpath):
        data = []
        element = etreepart.xpath(xpath, namespaces=self.ns)
        if element:
            for e in element:
                if e.text:
                    data.append(e.text)
                else:
                    data.append("")
        return data



import os
from setuptools import setup, find_packages
import pylom

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname), encoding='utf-8').read()

setup(
    name = pylom.__name__,
    version = pylom.__version__,
    author = "Wim Muskee",
    author_email = "w.muskee@kennisnet.nl",
    description = ("Library for parsing IMS-LOM xml records."),
    license = "MIT",
    keywords = "lom parsing xml",
    packages=find_packages(),
    long_description=read('README.md'),
    long_description_content_type="text/markdown",
    url="https://github.com/kennisnet/pylom",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries"
    ],
    install_requires = [
         'lxml'
         ]
)



from unittest import TestCase
from pylom.reader import LomReader

class ReaderElementsTestCase(TestCase):
    @classmethod
    def setUpClass(self):
        self.lomreader = LomReader()
        self.lomreader.parsePath("test/records/ims-complete.xml")

    def test_title(self):
        self.assertEqual(self.lomreader.lom["title"],"Bloodbath of B-R5RB")

    def test_catalogentry(self):
        self.assertEqual(self.lomreader.lom["catalogentry"][0]["catalog"], "URI")
        self.assertEqual(self.lomreader.lom["catalogentry"][0]["entry"], "https://en.wikipedia.org/wiki/Bloodbath_of_B-R5RB")
        self.assertEqual(self.lomreader.lom["catalogentry"][1]["catalog"], "URI")
        self.assertEqual(self.lomreader.lom["catalogentry"][1]["entry"], "https://www.wikidata.org/wiki/Q16987908")

    def test_language(self):
        self.assertIn("en",self.lomreader.lom["language"])

    def test_description(self):
        self.assertEqual(len(self.lomreader.lom["description"][0]),195)

    def test_keyword(self):
        self.assertIn("Eve Online",self.lomreader.lom["keyword"])
        self.assertIn("Halloween War",self.lomreader.lom["keyword"])

    def test_coverage(self):
        self.assertIn("YC 116",self.lomreader.lom["coverage"])

    def test_structure(self):
        self.assertEqual(self.lomreader.lom["structure"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["structure"]["value"],"hierarchical")

    def test_aggregationlevel(self):
        self.assertEqual(self.lomreader.lom["aggregationlevel"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["aggregationlevel"]["value"],"2")

    def test_version(self):
        self.assertEqual(self.lomreader.lom["version"],"1.0.0")

    def test_status(self):
        self.assertEqual(self.lomreader.lom["status"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["status"]["value"],"final")

    def test_contribute(self):
        self.assertEqual(self.lomreader.lom["contribute"][0]["role"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["contribute"][0]["role"]["value"],"publisher")
        self.assertEqual(self.lomreader.lom["contribute"][0]["entity"][0],"BEGIN:VCARD\nFN:Wikipedia\nEND:VCARD")
        self.assertEqual(self.lomreader.lom["contribute"][0]["date"]["datetime"],"2014-02-02T15:30:00Z")
        self.assertEqual(self.lomreader.lom["contribute"][1]["role"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["contribute"][1]["role"]["value"],"author")
        self.assertEqual(self.lomreader.lom["contribute"][1]["entity"][0],"BEGIN:VCARD\nFN:3family6\nEND:VCARD")
        self.assertEqual(self.lomreader.lom["contribute"][1]["date"]["datetime"],"2017-06-08T03:16:00Z")

    def test_metacatalogentry(self):
        self.assertEqual(self.lomreader.lom["metacatalogentry"][0]["catalog"], "URI")
        self.assertEqual(self.lomreader.lom["metacatalogentry"][0]["entry"], "https://github.com/kennisnet/pylom/blob/master/test/records/ims-complete.xml")

    def test_metacontribute(self):
        self.assertEqual(self.lomreader.lom["metacontribute"][0]["role"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["metacontribute"][0]["role"]["value"],"creator")
        self.assertEqual(self.lomreader.lom["metacontribute"][0]["entity"][0],"BEGIN:VCARD\nUID:https://github.com/wimmuskee\nEND:VCARD")
        self.assertEqual(self.lomreader.lom["metacontribute"][0]["date"]["datetime"],"2017")

    def test_metadatascheme(self):
        self.assertIn("LOMv1.0", self.lomreader.lom["metadatascheme"])

    def test_metalanguage(self):
        self.assertEqual(self.lomreader.lom["metalanguage"],"en")

    def test_format(self):
        self.assertIn("text/html", self.lomreader.lom["format"])

    def test_size(self):
        self.assertEqual(self.lomreader.lom["size"],"23450")

    def test_location(self):
        self.assertEqual(self.lomreader.lom["location"],"https://en.wikipedia.org/wiki/Bloodbath_of_B-R5RB")

    def test_duration(self):
        self.assertEqual(self.lomreader.lom["duration"]["datetime"],"PT0H16M")
        self.assertEqual(self.lomreader.lom["duration"]["description"],"technical reading time")

    def test_educational(self):
        self.assertEqual(self.lomreader.lom["educational"][0]["interactivitytype"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["interactivitytype"]["value"], "expositive")
        self.assertEqual(self.lomreader.lom["educational"][0]["learningresourcetype"][0]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["learningresourcetype"][0]["value"], "narrative text")
        self.assertEqual(self.lomreader.lom["educational"][0]["interactivitylevel"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["interactivitylevel"]["value"], "low")
        self.assertEqual(self.lomreader.lom["educational"][0]["semanticdensity"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["semanticdensity"]["value"], "high")
        self.assertEqual(self.lomreader.lom["educational"][0]["intendedenduserrole"][0]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["intendedenduserrole"][0]["value"], "learner")
        self.assertEqual(self.lomreader.lom["educational"][0]["context"][0]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["context"][0]["value"], "school")
        self.assertEqual(self.lomreader.lom["educational"][1]["context"][0]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][1]["context"][0]["value"], "higher education")
        self.assertIn("16-60", self.lomreader.lom["educational"][0]["typicalagerange"])
        self.assertEqual(self.lomreader.lom["educational"][0]["difficulty"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["educational"][0]["difficulty"]["value"], "difficult")
        self.assertEqual(self.lomreader.lom["educational"][0]["typicallearningtime"]["datetime"],"PT0H30M")
        self.assertEqual(self.lomreader.lom["educational"][0]["typicallearningtime"]["description"],"understanding reading time")
        self.assertEqual(self.lomreader.lom["educational"][0]["description"][0],"for individual reading")
        self.assertEqual(self.lomreader.lom["educational"][0]["language"][0],"en")

    def test_cost(self):
        self.assertEqual(self.lomreader.lom["cost"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["cost"]["value"],"no")

    def test_copyrightandotherrestrictions(self):
        self.assertEqual(self.lomreader.lom["copyrightandotherrestrictions"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["copyrightandotherrestrictions"]["value"],"yes")

    def test_copyrightdescription(self):
        self.assertEqual(len(self.lomreader.lom["copyrightdescription"]),47)

    def test_relation(self):
        self.assertEqual(self.lomreader.lom["relation"][0]["kind"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["relation"][0]["kind"]["value"], "references")
        self.assertEqual(self.lomreader.lom["relation"][0]["resource"]["description"][0], "Wikipedia article references")
        self.assertEqual(self.lomreader.lom["relation"][0]["resource"]["catalogentry"][0]["catalog"], "URI")
        self.assertEqual(self.lomreader.lom["relation"][0]["resource"]["catalogentry"][0]["entry"], "https://www.wired.com/gamelife/2014/02/eve-online-battle-of-b-r/")
        self.assertEqual(self.lomreader.lom["relation"][0]["resource"]["catalogentry"][1]["catalog"], "URI")
        self.assertEqual(self.lomreader.lom["relation"][0]["resource"]["catalogentry"][1]["entry"], "http://www.polygon.com/2014/1/30/5360208/Eve-Onlines-Bloodbath")
        self.assertEqual(self.lomreader.lom["relation"][1]["kind"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["relation"][1]["kind"]["value"], "isreferencedby")
        self.assertEqual(self.lomreader.lom["relation"][1]["resource"]["description"][0], "Wikipedia pages linking to this")
        self.assertEqual(self.lomreader.lom["relation"][1]["resource"]["catalogentry"][0]["catalog"], "URI")
        self.assertEqual(self.lomreader.lom["relation"][1]["resource"]["catalogentry"][0]["entry"], "https://www.wikidata.org/wiki/Q336177")

    def test_classification(self):
        self.assertEqual(self.lomreader.lom["classification"][0]["purpose"]["source"], "LOMv1.0")
        self.assertEqual(self.lomreader.lom["classification"][0]["purpose"]["value"], "idea")
        self.assertEqual(self.lomreader.lom["classification"][0]["taxonpath"][0]["source"], "https://en.wikipedia.org/wiki/Category:Articles")
        self.assertEqual(self.lomreader.lom["classification"][0]["taxonpath"][0]["taxon"][0]["id"], "Category:Games")
        self.assertEqual(self.lomreader.lom["classification"][0]["taxonpath"][0]["taxon"][0]["entry"], "Games")
        self.assertEqual(self.lomreader.lom["classification"][0]["taxonpath"][0]["taxon"][1]["id"], "Category:Space_MMORPGs")
        self.assertEqual(self.lomreader.lom["classification"][0]["taxonpath"][0]["taxon"][1]["entry"], "Space MMORPGs")
        self.assertEqual(self.lomreader.lom["classification"][0]["taxonpath"][0]["taxon"][2]["id"], "Category:Eve_Online")
        self.assertEqual(self.lomreader.lom["classification"][0]["taxonpath"][0]["taxon"][2]["entry"], "Eve Online")



from unittest import TestCase
from pylom.reader import LomReader

class FunctionsTestCase(TestCase):
    def setUp(self):
        self.lomreader = LomReader()

    def test_illegal_path(self):
        with self.assertRaises(RuntimeError):
            self.lomreader.parsePath("nopath")

    def test_fieldset(self):
        self.lomreader.parsePath("test/records/ims-complete.xml", ["title","structure"])
        self.assertEqual(self.lomreader.lom["title"],"Bloodbath of B-R5RB")
        self.assertEqual(self.lomreader.lom["structure"]["source"],"LOMv1.0")
        self.assertEqual(self.lomreader.lom["structure"]["value"],"hierarchical")
        self.assertEqual(self.lomreader.lom["size"],"")

    def test_stringread(self):
        with open("test/records/ims-complete.xml", "r") as f:
            lomstring = f.read()
        self.lomreader.parseString(lomstring)
        self.assertEqual(self.lomreader.lom["title"],"Bloodbath of B-R5RB")
